type options = {
	mutable input_filename: string;
	mutable no_use_utf16: bool;
	mutable help: bool};;

let options = {
	input_filename = "";
	no_use_utf16 = false;
	help = false};;

let rec loop_args i = (
	if i >= Array.length Sys.argv then () else
	let a = Sys.argv.(i) in
	if a = "-help" || a = "--help" then (
		options.help <- true;
		loop_args (i + 1)
	) else if a = "--no-utf16" then (
		options.no_use_utf16 <- true;
		loop_args (i + 1)
	) else if a.[0] = '-' then (
		failwith ("text2dfm: illegal option " ^ a)
	) else (
		options.input_filename <- a;
		loop_args (i + 1)
	)
) in
loop_args 1;;

if options.help then (
	print_string "usage: text2dfm options source_file\n";
	print_string "  --no-utf16 disable to output UTF-16.\n";
	print_string "  --help     show this message.\n";
	exit 0
);;

let input = (
	if options.input_filename <> "" then (
		open_in options.input_filename
	) else (
		stdin
	)
) in
set_binary_mode_out stdout true;
Dfm_text.text_to_resource ~no_utf16:options.no_use_utf16 input stdout;
if options.input_filename <> "" then close_in input;;
