type value_type =
	VaNull | VaList | VaInt8 | VaInt16 | VaInt32 | VaExtended |
	VaString | VaIdent | VaFalse | VaTrue | VaBinary | VaSet | VaLString |
	VaNil | VaCollection | VaSingle | VaCurrency | VaDate | VaWString |
	VaInt64 | VaUTF8String | VaDouble;;

val min_value: value_type;;
val max_value: value_type;;

type filer_flag = FfInherited | FfChildPos | FfInline;;
type filer_flags = int;;
val set_of_ff: filer_flag -> filer_flags;;

val filer_signature: string;;

type byte_array = (int, Bigarray.int8_unsigned_elt, Bigarray.c_layout) Bigarray.Array1.t;;

type variant = [
	| `bool of bool
	| `binary of byte_array
	| `char of char
	| `currency of float
	| `efloat of float * (int64 * int)
	| `float of float
	| `int of int
	| `int32 of int32
	| `int64 of int64
	| `nil
	| `null
	| `sfloat of float
	| `string of string
	| `tag of string
	| `time of float
	| `wchar of int
	| `wstring of Unicode.utf16_string];;

val datetime_of_unixtime: float -> float;;
val unixtime_of_datetime: float -> float;;

val extended_of_float: float -> int64 * int;;
val float_of_extended: int64 * int -> float;;

module Writer: sig
	type write_handler = string -> int -> int -> unit;;
	val write_raw_byte: string -> write_handler -> int -> unit
	val write_raw_int8: write_handler -> int -> unit
	val write_raw_int16: write_handler -> int -> unit
	val write_raw_int32: write_handler -> int32 -> unit
	val write_raw_int64: write_handler -> int64 -> unit
	val write_raw_extended_image: write_handler -> int64 * int -> unit
	val write_raw_extended: write_handler -> float -> unit
	val write_raw_short_string: write_handler -> string -> unit
	val write_raw_zero_terminated_string: write_handler -> string -> unit
	val write_resource_header: write_handler -> string -> int -> unit
	val write_value: write_handler -> value_type -> unit
	val write_int8: write_handler -> int -> unit
	val write_int16: write_handler -> int -> unit
	val write_int32: write_handler -> int32 -> unit
	val write_int64: write_handler -> int64 -> unit
	val write_int: write_handler -> int -> unit
	val write_string: write_handler -> string -> unit
	val write_wide_string: write_handler -> Unicode.utf16_string -> unit
	val write_char: write_handler -> char -> unit
	val write_wchar: write_handler -> int -> unit
	val write_bool: write_handler -> bool -> unit
	val write_ident: write_handler -> string -> unit
	val write_single: write_handler -> float -> unit
	val write_double: write_handler -> float -> unit
	val write_extended_image: write_handler -> int64 * int -> unit
	val write_extended: write_handler -> float -> unit
	val write_currency: write_handler -> float -> unit
	val write_date: write_handler -> float -> unit
	val write_binary: write_handler -> byte_array -> unit
	val write_nil: write_handler -> unit
	val write_null: write_handler -> unit
	val write_property_name: write_handler -> string -> unit
	val write_list_begin: write_handler -> unit
	val write_list_end: write_handler -> unit
	val write_collection_begin: write_handler -> unit
	val write_collection_end: write_handler -> unit
	val write_set_begin: write_handler -> unit
	val write_set_end: write_handler -> unit
	val write_set_element: write_handler -> [ `char of char | `int of int | `tag of string | `wchar of int] -> unit
	val write_signature: write_handler -> unit
	val write_component: write_handler -> filer_flags -> int -> string -> string -> unit
	val write_variant: write_handler -> variant -> unit
	val create:
		?resource_header:bool ->
		string ->
		out_channel ->
		[> `writing of
			(unit -> unit) list ref *
			(string ->
				[< variant
				| `end_of_list
				| `end_of_record
				| `list of [< `any | `record]
				| `none
				| `record
				| `some] ->
				unit)]
end;;

module Reader: sig
	type read_handler = string -> int -> int -> int;;
	val read_raw_byte: string -> read_handler -> int
	val read_raw_int8: read_handler -> int
	val read_raw_int16: read_handler -> int
	val read_raw_int32: read_handler -> int32
	val read_raw_int64: read_handler -> int64
	val read_raw_extended: read_handler -> float * (int64 * int)
	val read_raw_date: read_handler -> float
	val read_raw_short_string: read_handler -> string
	val read_raw_zero_terminated_string: read_handler -> string
	val read_raw_long_string: read_handler -> string
	val read_raw_wide_string: read_handler -> Unicode.utf16_string
	val read_raw_binary: read_handler -> byte_array
	val read_value: read_handler -> value_type
	val read_int: read_handler -> int64
	val read_set_element: read_handler -> [ `char of char | `int of int | `tag of string | `wchar of int] option
	val read_list: read_handler -> int option
	val read_variant: read_handler -> [`list | `set | `collection | variant]
	val read_property_name: read_handler -> string option
	val read_resource_header: read_handler -> string
	val read_signature: read_handler -> unit
	val read_component: read_handler -> (filer_flags * int * string * string) option
	val create:
		?resource_header:bool ->
		string ->
		in_channel ->
		[> `reading of
			string ref *
			[> `binary of byte_array
			| `bool of bool
			| `char of char
			| `end_of_list
			| `end_of_record
			| `float of float
			| `int of int
			| `int32 of int32
			| `int64 of int64
			| `list of [> `any]
			| `nil
			| `null
			| `record
			| `string of string
			| `tag of string
			| `time of float] ref *
			(unit -> unit) *
			(float -> float)]
end;;
