val extended_of_string: (string -> float * (int64 * int)) ref
val string_of_extended: (float * (int64 * int) -> string) ref
module Parser: sig
	type float_type = Single | Date | Extended of (int64 * int) | Currency
	type token =
		| ToEOF
		| ToSymbol of string
		| ToString of string
		| ToInteger of int64
		| ToFloat of float * float_type
		| ToWString of Unicode.utf16_string
		| ToChar of char
		| ToBinary of Dfm.byte_array
	exception Error of int * string
	type t
	val error: t -> string -> 'a
	val skip_blanks: t -> unit
	val next_token: t -> unit
	val create: char Stream.t -> t
	val source_line: t -> int
	val token: t -> token
	val token_symbol_is: t -> string -> bool
	val token_char_is: t -> char -> bool
end
val internal_binary_to_text:
	?name:string ref ->
	?linefeed:string ->
	?indent:string ->
	?utf8:bool ->
	(string -> int -> int -> int) ->
	(string -> unit) ->
	unit
val binary_to_text:
	?name:string ref ->
	?linefeed:string ->
	?indent:string ->
	?utf8:bool ->
	in_channel ->
	out_channel ->
	unit
val resource_to_text:
	?resource_name:string ref ->
	?name:string ref ->
	?linefeed:string ->
	?indent:string ->
	?utf8:bool ->
	in_channel ->
	out_channel ->
	unit
val internal_text_to_binary:
	?name:string ref ->
	?no_utf16:bool ->
	char Stream.t ->
	(string -> int -> int -> unit) ->
	unit
val text_to_binary:
	?name:string ref ->
	?no_utf16:bool ->
	in_channel ->
	out_channel ->
	unit
val text_to_resource:
	?resource_name:string ->
	?name:string ref ->
	?no_utf16:bool ->
	in_channel ->
	out_channel ->
	unit
