open Format;;

let pp_print_abstr f (_: [> ]) = pp_print_string f "<abstr>";;

let create_abstr ?(pp: formatter -> 'v -> unit = pp_print_abstr) (f: formatter)
	: [> `writing of (unit -> unit) list ref * (string -> 'v -> unit)] =
(
	let state = ref [] in
	`writing (ref [], (fun key value ->
		begin match value with
		| `end_of_record | `end_of_list ->
			begin match !state with 
			| s :: sr ->
				begin match s with
				| `record1 | `record2 ->
					pp_print_string f "}"
				| `list1 | `list2 ->
					pp_print_string f "]"
				| `some -> ()
				end;
				pp_close_box f ();
				begin match sr with
				| `record2 :: _ -> pp_close_box f ()
				| [] -> pp_print_cut f ()
				| _ -> ()
				end;
				state := sr
			| [] -> assert false
			end
		| _ ->
			begin match !state with
			| (`record1 | `record2) :: _ ->
				if key = "" then failwith "create_abstr:key was required"
			| (`list1 | `list2 | `some) :: _ | [] ->
				if key <> "" then failwith "create_abstr:key was not required"
			end;
			begin match !state with
			| `record1 :: sr ->
				state := `record2 :: sr
			| `list1 :: sr ->
				state := `list2 :: sr
			| (`record2 | `list2) :: _ ->
				pp_print_string f ";";
				pp_print_space f ()
			| `some :: _ | [] -> ()
			end;
			begin match !state with
			| `record2 :: _ ->
				pp_open_box f (String.length key + 3);
				pp_print_string f key;
				pp_print_string f " = ";
			| `some :: sr ->
				state := sr
			| (`list2 | `record1 | `list1) :: _ | [] -> ()
			end;
			begin match value with
			| `record ->
				pp_print_string f "{";
				pp_open_hvbox f 0;
				state := `record1 :: !state
			| `list _ ->
				pp_print_string f "[";
				pp_open_hvbox f 0;
				state := `list1 :: !state
			| a ->
				begin match a with
				| `none ->
					pp_print_string f "None"
				| `some ->
					pp_print_string f "Some ";
					state := `some :: !state
				| a ->
					pp f a;
				end;
				begin match !state with
				| `record2 :: _ ->
					pp_close_box f ()
				| _ -> ()
				end
			end
		end
	))
);;

type pervasives_variant = [
	| `bool of bool
	| `char of char
	| `float of float
	| `int of int
	| `string of string
	| `tag of string];;

let pp_pervasives ?(pp = pp_print_abstr) f v = (
	match v with
	| `bool v ->
		pp_print_bool f v
	| `char v ->
		pp_print_char f '\'';
		pp_print_string f (Char.escaped v);
		pp_print_char f '\''
	| `float v ->
		pp_print_float f v
	| `int v ->
		pp_print_int f v
	| `string v ->
		pp_print_char f '\"';
		pp_print_string f (String.escaped v);
		pp_print_char f '\"'
	| `tag v ->
		pp_print_string f v
	| a ->
		pp f a
);;

let pp_serialize ?(pp = pp_print_abstr) f v = (
	match v with
	| `record -> pp_print_string f "`record"
	| `list a ->
		pp_print_string f "`list ";
		begin match a with
		| `any -> pp_print_string f "`any"
		| `record -> pp_print_string f "`record"
		end
	| `end_of_record -> pp_print_string f "`end_of_record"
	| `end_of_list -> pp_print_string f "`end_of_list"
	| a -> pp f a
);;

let create ?(pp = pp_pervasives ~pp:pp_print_abstr) = create_abstr ~pp;;
